@@ -20,32 +20,58 @@ from ..tasks import ExternalTask, Task
 __all__ = ["QChem", "QChemInput", "QChemOutput"]
 
 
-# FIXME: make it possible to take user-defined charges & multiplicities, both for the overall structure and for fragments?
 class QChemInput:
-    def __init__(self, *molecules: mtr.Molecule, settings: mtr.Settings,) -> None:
+    def __init__(
+        self,
+        *molecules: mtr.Molecule,
+        settings: mtr.Settings,
+        charges: Optional[Iterable[int]] = None,
+        multiplicities: Optional[Iterable[int]] = None,
+        total_charge: Optional[int] = None,
+        total_multiplicity: Optional[int] = None,
+        string: Optional[str] = None,
+    ) -> None:
         self.molecules = molecules
         self.settings = settings
+        self.charges = charges or [m.charge for m in self.molecules]
+        self.multiplicities = multiplicities or [m.multiplicity for m in self.molecules]
+        # FIXME: seems more rigorously correct to determine the total charge and multiplicity from combined structures rather than just adding individual charges & multiplicities
+        self.total_charge = total_charge or sum(self.charges)
+        self.total_multiplicity = total_multiplicity or self.total_charge % 2 + 1
+        self.string = string
 
     def write(self, filepath: str) -> None:
         with open(mtr.expand(filepath), "w") as f:
             f.write(str(self))
 
     def __str__(self) -> str:
-        # FIXME: seems more rigorously correct to determine the total charge and multiplicity from combined structures rather than just adding individual charges & multiplicities
-        charge = sum(m.charge for m in self.molecules)
-        multiplicity = charge % 2 + 1
+        if self.string is not None:
+            return self.string
+
         total_block = (
-            f"  {charge} {multiplicity}\n--\n" if len(self.molecules) > 1 else ""
-        )
-        molecule_block = total_block + "\n--\n".join(
-            _molecule_to_structure_block(molecule=m) for m in self.molecules
+            f"  {self.total_charge} {self.total_multiplicity}\n--\n"
+            if len(self.molecules) > 1
+            else ""
         )
 
-        return f"$molecule\n{molecule_block}\n$end\n" + "\n".join(
+        molecule_block = total_block + "\n--\n".join(
+            _molecule_to_structure_block(molecule=m, charge=c, multiplicity=mult)
+            for m, c, mult in zip(self.molecules, self.charges, self.multiplicities)
+        )
+
+        if len(molecule_block) == 0:
+            molecule_block = "  read"
+
+        return f"$molecule\n{molecule_block}\n$end\n\n" + "\n".join(
             _block_to_str(block_name=block_name, block_params=block_params)
             for block_name, block_params in self.settings.d.items()
         )
 
+    def __add__(self, other: mtr.QChemInput) -> str:
+        return QChemInput(
+            string=str(self) + "\n@@@\n\n" + str(other), settings=mtr.Settings()
+        )
+
 
 def _molecule_to_structure_block(
     molecule: mtr.Molecule,
@@ -178,6 +204,15 @@ class QChemOutput:
 
         return mtr.Polarizability(polarizability_tensor=pol * mtr.au_volume)
 
+    @property
+    def structure(self) -> mtr.Structure:
+        coords = self.cclib_out.atomcoords[-1, :, :] * mtr.angstrom
+        zs = self.cclib_out.atomnos
+
+        atoms = (mtr.Atom(element=Z, position=p) for Z, p in zip(zs, coords))
+
+        return mtr.Structure(*atoms)
+
     # def rttddft(self, lines: str):  # FIXME: not sure how to annotate type hint
     #     s = r"ITER:\s*(\d*)\s*T:\s*(\d*\.\d*)\(fs\)\s*dt\s*(\d*\.\d*)\(fs\)\s*Hr/Ps:\s*(\d*\.\d*)\s*-\s*Lpsd/Rem\.:\s*(\d*\.\d*),\s*([^\s]*)\s*\(min\)\s*Tr\.Dev:\s*(\d*\.\d*)\s*Hrm:\s*(\d*\.\d*)\s*Enrgy:\s*(-?\d*\.\d*)\s*Entr:\s*(-?\d*.\d*)\s*Fld\s*(\d*)\s*NFk:\s*(\d*)\s*Mu\s*(-?\d*\.\d*e?-?\d*)\s*(-?\d*\.\d*e?-?\d*)\s*(-?\d*\.\d*e?-?\d*)"
     #     pattern = re.compile(s)
@@ -251,8 +286,8 @@ class QChemOutput:
 
     @property
     def electronic_excitations(self) -> mtr.ExcitationSpectrum:
-        engs = mtr.h * mtr.c * (self.cclib_out.etenergies / mtr.cm)
-        engs = engs.convert(mtr.eV)
+        engs = self.cclib_out.etenergies / mtr.cm
+        engs = (mtr.h * mtr.c * engs).convert(mtr.eV)
         excitations = tuple(
             mtr.Excitation(
                 energy=eng, oscillator_strength=osc, symmetry=sym, contributions=cont
@@ -353,9 +388,13 @@ class QChem(Engine):
         num_processors: Optional[int] = None,
         num_threads: Optional[int] = None,
         arguments: Optional[Iterable[str]] = None,
+        save: Optional[bool] = False,
+        savename: Optional[str] = None,
     ) -> None:
         self.scratch_dir = mtr.expand(scratch_dir) if scratch_dir is not None else None
         self.qcenv = shlex.quote(mtr.expand(qcenv)) if qcenv is not None else None
+        self.save = save
+        self.savename = savename
         super().__init__(
             executable=executable,
             num_processors=num_processors,
@@ -396,12 +435,16 @@ class QChem(Engine):
         arguments: Optional[Iterable[str]] = None,
     ) -> str:
         cmd = [self.executable]
+        if self.save:
+            cmd.append("-save")
         if self.arguments is not None:
             cmd.extend(list(self.arguments) + list(arguments or []))
         if self.num_processors is not None:
             cmd.append(f"-np {self.num_processors}")
         if self.num_threads is not None:
             cmd.append(f"-nt {self.num_threads}")
+        if self.save and self.savename:
+            cmd.append(f" {self.savename}")
 
         # FIXME: shlex.quote should be used but it doesn't work...
         return shlex.split(" ".join(cmd + [inp, out]))
@@ -415,6 +458,7 @@ class QChem(Engine):
         return QChemAIMD(engine=self, io=io, handlers=handlers, name=name)
 
     def koopman_error(
+        self,
         gs_io: mtr.IO,
         cation_io: mtr.IO,
         anion_io: mtr.IO,
@@ -430,37 +474,38 @@ class QChem(Engine):
             name=name,
         )
 
-    def koopman_error_lpscf(
-        gs_io: mtr.IO,
-        cation_io: mtr.IO,
-        anion_io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> QChemKoopmanErrorLPSCF:
-        return QChemKoopmanErrorLPSCF(
-            engine=self,
-            gs_io=gs_io,
-            cation_io=cation_io,
-            anion_io=anion_io,
-            handlers=handlers,
-            name=name,
-        )
+    # def koopman_error_lpscf(
+    #     self,
+    #     gs_io: mtr.IO,
+    #     cation_io: mtr.IO,
+    #     anion_io: mtr.IO,
+    #     handlers: Optional[Iterable[mtr.Handler]] = None,
+    #     name: Optional[str] = None,
+    # ) -> QChemKoopmanErrorLPSCF:
+    #     return QChemKoopmanErrorLPSCF(
+    #         engine=self,
+    #         gs_io=gs_io,
+    #         cation_io=cation_io,
+    #         anion_io=anion_io,
+    #         handlers=handlers,
+    #         name=name,
+    #     )
 
-    def lpscf(
-        self,
-        io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> QChemLPSCF:
-        return QChemLPSCF(engine=self, io=io, handlers=handlers, name=name)
+    # def lpscf(
+    #     self,
+    #     io: mtr.IO,
+    #     handlers: Optional[Iterable[mtr.Handler]] = None,
+    #     name: Optional[str] = None,
+    # ) -> QChemLPSCF:
+    #     return QChemLPSCF(engine=self, io=io, handlers=handlers, name=name)
 
-    def lpscfrs(
-        self,
-        io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> QChemLPSCFRS:
-        return QChemLPSCFRS(engine=self, io=io, handlers=handlers, name=name)
+    # def lpscfrs(
+    #     self,
+    #     io: mtr.IO,
+    #     handlers: Optional[Iterable[mtr.Handler]] = None,
+    #     name: Optional[str] = None,
+    # ) -> QChemLPSCFRS:
+    #     return QChemLPSCFRS(engine=self, io=io, handlers=handlers, name=name)
 
     def lrtddft(
         self,
@@ -470,6 +515,14 @@ class QChem(Engine):
     ) -> QChemLRTDDFT:
         return QChemLRTDDFT(engine=self, io=io, handlers=handlers, name=name)
 
+    def lrtddft_plot_ntos(
+        self,
+        io: mtr.IO,
+        handlers: Optional[Iterable[mtr.Handler]] = None,
+        name: Optional[str] = None,
+    ) -> QChemLRTDDFTPlotNTOs:
+        return QChemLRTDDFTPlotNTOs(engine=self, io=io, handlers=handlers, name=name)
+
     def minimize_koopman_error(
         self,
         io: mtr.IO,
@@ -480,15 +533,15 @@ class QChem(Engine):
             engine=self, io=io, handlers=handlers, name=name
         )
 
-    def minimize_koopman_error_lpscf(
-        self,
-        io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> QChemMinimizeKoopmanErrorLPSCF:
-        return QChemMinimizeKoopmanErrorLPSCF(
-            engine=self, io=io, handlers=handlers, name=name
-        )
+    # def minimize_koopman_error_lpscf(
+    #     self,
+    #     io: mtr.IO,
+    #     handlers: Optional[Iterable[mtr.Handler]] = None,
+    #     name: Optional[str] = None,
+    # ) -> QChemMinimizeKoopmanErrorLPSCF:
+    #     return QChemMinimizeKoopmanErrorLPSCF(
+    #         engine=self, io=io, handlers=handlers, name=name
+    #     )
 
     def optimize(
         self,
@@ -548,8 +601,7 @@ class QChemBaseTask(ExternalTask):
     ) -> Any:
         s = mtr.Settings() if settings is None else copy.deepcopy(settings)
 
-        # FIXME: this is essentially a hotpatch to handle fragments - come up with something more elegant/sensible ASAP
-        inp = mtr.QChemInput(molecule=molecule, settings=self.defaults(s),)
+        inp = mtr.QChemInput(molecule, settings=self.defaults(s))
 
         with self.io() as io:
             inp.write(io.inp)
@@ -596,68 +648,65 @@ class QChemAIMD(QChemBaseTask):
         return settings
 
 
-class QChemLPSCF(QChemBaseTask):
-    def parse(self, output: str) -> Any:
-        try:
-            energy = cclib.io.ccread(output).scfenergies * mtr.eV
-        except AttributeError:
-            energy = None
+# class QChemLPSCF(QChemBaseTask):
+#     def parse(self, output: str) -> Any:
+#         try:
+#             energy = cclib.io.ccread(output).scfenergies * mtr.eV
+#         except AttributeError:
+#             energy = None
 
-        return energy
+#         return energy
 
-    def defaults(self, settings: mtr.Settings) -> mtr.Settings:
-        if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
-            settings["rem", "exchange"] = "HF"
-        if ("rem", "basis") not in settings:
-            settings["rem", "basis"] = "3-21G"
-        if ("rem", "jobtype") not in settings:
-            settings["rem", "jobtype"] = "sp"
-        if ("rem", "frgm_method") not in settings:
-            settings["rem", "frgm_method"] = "stoll"
-        if ("rem_frgm", "scf_convergence") not in settings:
-            settings["rem_frgm", "scf_convergence"] = 2
-        if ("rem_frgm", "thresh") not in settings:
-            settings["rem_frgm", "thresh"] = 5
+#     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
+#         if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
+#             settings["rem", "exchange"] = "HF"
+#         if ("rem", "basis") not in settings:
+#             settings["rem", "basis"] = "3-21G"
+#         if ("rem", "jobtype") not in settings:
+#             settings["rem", "jobtype"] = "sp"
+#         if ("rem", "frgm_method") not in settings:
+#             settings["rem", "frgm_method"] = "stoll"
+#         if ("rem_frgm", "scf_convergence") not in settings:
+#             settings["rem_frgm", "scf_convergence"] = 2
+#         if ("rem_frgm", "thresh") not in settings:
+#             settings["rem_frgm", "thresh"] = 5
 
-        return settings
+#         return settings
 
-    def run(
-        self, *fragments: mtr.Molecule, settings: Optional[mtr.Settings] = None,
-    ) -> Any:
-        s = mtr.Settings() if settings is None else copy.deepcopy(settings)
+#     def run(
+#         self, *fragments: mtr.Molecule, settings: Optional[mtr.Settings] = None,
+#     ) -> Any:
+#         s = mtr.Settings() if settings is None else copy.deepcopy(settings)
 
-        # FIXME: this is essentially a hotpatch to handle fragments - come up with something more elegant/sensible ASAP
-        inp = mtr.QChemInput(
-            molecule=mtr.QChemFragments(structures=fragments),
-            settings=self.defaults(s),
-        )
+#         # FIXME: this is essentially a hotpatch to handle fragments - come up with something more elegant/sensible ASAP
+#         inp = mtr.QChemInput(*fragments, settings=self.defaults(s),)
 
-        with self.io() as io:
-            inp.write(io.inp)
+#         with self.io() as io:
+#             inp.write(io.inp)
 
-            self.engine.execute(self.io)
+#             self.engine.execute(self.io)
 
-            return self.parse(io.out)
+#             return self.parse(io.out)
 
 
-class QChemLPSCFRS(QChemBaseTask):
-    def defaults(self, settings: mtr.Settings) -> mtr.Settings:
-        if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
-            settings["rem", "exchange"] = "HF"
-        if ("rem", "basis") not in settings:
-            settings["rem", "basis"] = "3-21G"
-        if ("rem", "jobtype") not in settings:
-            settings["rem", "jobtype"] = "sp"
-        if ("rem", "frgm_method") not in settings:
-            settings["rem", "frgm_method"] = "stoll"
-        if ("rem_frgm", "scf_convergence") not in settings:
-            settings["rem_frgm", "scf_convergence"] = 2
-        if ("rem_frgm", "thresh") not in settings:
-            settings["rem_frgm", "thresh"] = 5
-        if ("rem_frgm", "frgm_lpcorr") not in settings:
-            settings["rem_frgm", "frgm_lpcorr"] = "rs_exact_scf"
+# class QChemLPSCFRS(QChemBaseTask):
+#     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
+#         if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
+#             settings["rem", "exchange"] = "HF"
+#         if ("rem", "basis") not in settings:
+#             settings["rem", "basis"] = "3-21G"
+#         if ("rem", "jobtype") not in settings:
+#             settings["rem", "jobtype"] = "sp"
+#         if ("rem", "frgm_method") not in settings:
+#             settings["rem", "frgm_method"] = "stoll"
+#         if ("rem_frgm", "scf_convergence") not in settings:
+#             settings["rem_frgm", "scf_convergence"] = 2
+#         if ("rem_frgm", "thresh") not in settings:
+#             settings["rem_frgm", "thresh"] = 5
+#         if ("rem_frgm", "frgm_lpcorr") not in settings:
+#             settings["rem_frgm", "frgm_lpcorr"] = "rs_exact_scf"
 
-        return settings
+#         return settings
 
 
 class QChemKoopmanError(Task):
@@ -703,27 +752,22 @@ class QChemKoopmanError(Task):
         cation_sp = self.engine.single_point(self.cation_io, name="cation")
         anion_sp = self.engine.single_point(self.anion_io, name="anion")
 
+        cation = copy.deepcopy(molecule)
+        cation.charge += 1
+        cation.multiplicity = cation.multiplicity % 2 + 1
+
+        anion = copy.deepcopy(molecule)
+        anion.charge -= 1
+        anion.multiplicity = anion.multiplicity % 2 + 1
+
         neutral_sp.requires(
-            molecule=mtr.QChemStructure(
-                molecule, charge=molecule.charge, multiplicity=molecule.multiplicity
-            ),
-            settings=input_settings,
+            molecule=molecule, settings=input_settings,
         )
         cation_sp.requires(
-            structure=mtr.QChemStructure(
-                structure,
-                charge=molecule.charge + 1,
-                multiplicity=(molecule.multiplicity + 1) % 2,
-            ),
-            settings=input_settings,
+            molecule=cation, settings=input_settings,
         )
         anion_sp.requires(
-            structure=mtr.QChemStructure(
-                structure,
-                charge=molecule.charge - 1,
-                multiplicity=(molecule.multiplicity + 1) % 2,
-            ),
-            settings=input_settings,
+            molecule=anion, settings=input_settings,
         )
 
         wf = Workflow(neutral_sp, cation_sp, anion_sp)
@@ -735,7 +779,7 @@ class QChemKoopmanError(Task):
         anion = out["anion"]
 
         ea = neutral - anion
-        ip = cation - energy
+        ip = cation - neutral
 
         J_squared = (ip + homo) ** 2
         if ea > 0 * ea.unit:
@@ -744,115 +788,109 @@ class QChemKoopmanError(Task):
         return np.sqrt(J_squared.convert(mtr.eV ** 2).value.item()) * mtr.eV
 
 
-class QChemKoopmanErrorLPSCF(Task):
-    def __init__(
-        self,
-        engine: mtr.Engine,
-        gs_io: mtr.IO,
-        cation_io: mtr.IO,
-        anion_io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> None:
-        super().__init__(
-            (engine.num_threads or 1) * (engine.num_processors or 1),
-            handlers=handlers,
-            name=name,
-        )
-        self.engine = engine
-        self.gs_io = gs_io
-        self.cation_io = cation_io
-        self.anion_io = anion_io
+# class QChemKoopmanErrorLPSCF(Task):
+#     def __init__(
+#         self,
+#         engine: mtr.Engine,
+#         gs_io: mtr.IO,
+#         cation_io: mtr.IO,
+#         anion_io: mtr.IO,
+#         handlers: Optional[Iterable[mtr.Handler]] = None,
+#         name: Optional[str] = None,
+#     ) -> None:
+#         super().__init__(
+#             (engine.num_threads or 1) * (engine.num_processors or 1),
+#             handlers=handlers,
+#             name=name,
+#         )
+#         self.engine = engine
+#         self.gs_io = gs_io
+#         self.cation_io = cation_io
+#         self.anion_io = anion_io
 
-    def defaults(self, settings: mtr.Settings) -> mtr.Settings:
-        if ("rem", "exchange") not in settings and ("rem", "method") not in settings:
-            settings["rem", "exchange"] = "hf"
-        if ("rem", "basis") not in settings:
-            settings["rem", "basis"] = "3-21G"
-        if ("rem", "jobtype") not in settings:
-            settings["rem", "jobtype"] = "sp"
+#     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
+#         if ("rem", "exchange") not in settings and ("rem", "method") not in settings:
+#             settings["rem", "exchange"] = "hf"
+#         if ("rem", "basis") not in settings:
+#             settings["rem", "basis"] = "3-21G"
+#         if ("rem", "jobtype") not in settings:
+#             settings["rem", "jobtype"] = "sp"
 
-        return settings
+#         return settings
 
-    def run(
-        self,
-        *fragments: mtr.Structure,
-        active_fragment: int,
-        settings: Optional[mtr.Settings] = None,
-        num_consumers: Optional[int] = 1,
-    ) -> float:
-        s = mtr.Settings() if settings is None else copy.deepcopy(settings)
+#     def run(
+#         self,
+#         *fragments: mtr.Structure,
+#         active_fragment: int,
+#         settings: Optional[mtr.Settings] = None,
+#         num_consumers: Optional[int] = 1,
+#     ) -> float:
+#         s = mtr.Settings() if settings is None else copy.deepcopy(settings)
 
-        # if ("structure", "charge") not in settings:
-        #     settings["structure", "charge"] = 0
-        #     # FIXME: rather than guessing 0, use rdkit.Chem.rdmolops.GetFormalCharge?
-        # if ("structure", "multiplicity") not in settings:
-        #     settings["structure", "multiplicity"] = 1
+#         gs_charges = [0] * len(fragments)
+#         gs_multiplicities = [1] * len(fragments)
 
-        gs_charges = [0] * len(fragments)
-        gs_multiplicities = [1] * len(fragments)
+#         cation_charges = gs_charges.copy()
+#         cation_multiplicities = gs_multiplicities.copy()
+#         cation_charges[active_fragment] += 1
+#         cation_multiplicities[active_fragment] = 2
 
-        cation_charges = gs_charges.copy()
-        cation_multiplicities = gs_multiplicities.copy()
-        cation_charges[active_fragment] += 1
-        cation_multiplicities[active_fragment] = 2
+#         anion_charges = gs_charges.copy()
+#         anion_multiplicities = gs_multiplicities.copy()
+#         anion_charges[active_fragment] -= 1
+#         anion_multiplicities[active_fragment] = 2
 
-        anion_charges = gs_charges.copy()
-        anion_multiplicities = gs_multiplicities.copy()
-        anion_charges[active_fragment] -= 1
-        anion_multiplicities[active_fragment] = 2
+#         input_settings = self.defaults(s)
 
-        input_settings = self.defaults(s)
+#         gs = self.engine.single_point_frontier(self.gs_io, name="gs")
+#         gs_structure = mtr.QChemFragments(
+#             fragments,
+#             gs_charges,
+#             gs_multiplicities,
+#             total_charge=sum(gs_charges),
+#             total_multiplicity=1,
+#         )
+#         gs.requires(structure=gs_structure, settings=input_settings)
 
-        gs = QChemSinglePointFrontier(self.engine, self.gs_io, name="gs")
-        gs_structure = mtr.QChemFragments(
-            fragments,
-            gs_charges,
-            gs_multiplicities,
-            total_charge=sum(gs_charges),
-            total_multiplicity=1,
-        )
-        gs.requires(structure=gs_structure, settings=input_settings)
+#         cation = self.engine.single_point(self.cation_io, name="cation")
+#         cation_structure = mtr.QChemFragments(
+#             fragments,
+#             cation_charges,
+#             cation_multiplicities,
+#             total_charge=sum(cation_charges),
+#             total_multiplicity=2,
+#         )
+#         cation.requires(structure=cation_structure, settings=input_settings)
 
-        cation = QChemSinglePoint(self.engine, self.cation_io, name="cation")
-        cation_structure = mtr.QChemFragments(
-            fragments,
-            cation_charges,
-            cation_multiplicities,
-            total_charge=sum(cation_charges),
-            total_multiplicity=2,
-        )
-        cation.requires(structure=cation_structure, settings=input_settings)
+#         anion = self.engine.single_point(self.anion_io, name="anion")
+#         anion_structure = mtr.QChemFragments(
+#             fragments,
+#             anion_charges,
+#             anion_multiplicities,
+#             total_charge=sum(anion_charges),
+#             total_multiplicity=2,
+#         )
+#         anion.requires(structure=anion_structure, settings=input_settings)
 
-        anion = QChemSinglePoint(self.engine, self.anion_io, name="anion")
-        anion_structure = mtr.QChemFragments(
-            fragments,
-            anion_charges,
-            anion_multiplicities,
-            total_charge=sum(anion_charges),
-            total_multiplicity=2,
-        )
-        anion.requires(structure=anion_structure, settings=input_settings)
+#         wf = Workflow(gs, cation, anion)
 
-        wf = Workflow(gs, cation, anion)
+#         out = wf.run(available_cores=self.num_cores, num_consumers=num_consumers)
 
-        out = wf.run(available_cores=self.num_cores, num_consumers=num_consumers)
+#         energy, homo, lumo = out["gs"]
+#         cation = out["cation"]
+#         anion = out["anion"]
 
-        energy, homo, lumo = out["gs"]
-        cation = out["cation"]
-        anion = out["anion"]
+#         ea = energy - anion
+#         ip = cation - energy
 
-        ea = energy - anion
-        ip = cation - energy
+#         J_squared = (ea + lumo) ** 2 + (ip + homo) ** 2
 
-        J_squared = (ea + lumo) ** 2 + (ip + homo) ** 2
-
-        return np.sqrt(J_squared.convert(mtr.eV ** 2).value.item())
+#         return np.sqrt(J_squared.convert(mtr.eV ** 2).value.item())
 
 
 class QChemLRTDDFT(QChemBaseTask):
     def parse(self, output: str) -> Any:
-        return mtr.QChemOutput(filepath=output).get("electronic_excitations")
+        return mtr.QChemOutput(filepath=output).electronic_excitations
 
     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
         if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
@@ -866,10 +904,78 @@ class QChemLRTDDFT(QChemBaseTask):
         if ("rem", "cis_triplets") not in settings:
             settings["rem", "cis_triplets"] = False
         if ("rem", "rpa") not in settings:
-            settings["rem", "rpa"] = False
+            settings["rem", "rpa"] = True
 
         return settings
 
+    def run(
+        self, molecule: mtr.Molecule, settings: Optional[mtr.Settings] = None
+    ) -> mtr.Molecule:
+        molecule.electronic_excitations = super().run(molecule, settings)
+        return molecule
+
+
+class QChemLRTDDFTPlotNTOs(QChemBaseTask):
+    def parse(self, output: str) -> Any:
+        return mtr.QChemOutput(filepath=output).electronic_excitations
+
+    def defaults(self, settings: mtr.Settings) -> mtr.Settings:
+        if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
+            settings["rem", "exchange"] = "HF"
+        if ("rem", "basis") not in settings:
+            settings["rem", "basis"] = "3-21G"
+        if ("rem", "cis_n_roots") not in settings:
+            settings["rem", "cis_n_roots"] = 1
+        if ("rem", "cis_singlets") not in settings:
+            settings["rem", "cis_singlets"] = True
+        if ("rem", "cis_triplets") not in settings:
+            settings["rem", "cis_triplets"] = False
+        if ("rem", "rpa") not in settings:
+            settings["rem", "rpa"] = True
+
+        return settings
+
+    def run(
+        self,
+        molecule: mtr.Molecule,
+        settings: Optional[mtr.Settings] = None,
+        n_x: Optional[int] = 50,
+        n_y: Optional[int] = 50,
+        n_z: Optional[int] = 50,
+        num_nto_pairs: Optional[int] = 3,
+    ) -> mtr.Molecule:
+        s = mtr.Settings() if settings is None else copy.deepcopy(settings)
+        s = self.defaults(s)
+
+        inp = mtr.QChemInput(molecule, settings=s)
+
+        n_alpha = int(round((sum(molecule.atomic_numbers) + molecule.charge) / 2))
+
+        for i in range(s["rem", "cis_n_roots"]):
+            _s = copy.deepcopy(s)
+            _s["rem", "scf_guess"] = "read"
+            _s["rem", "skip_cis_rpa"] = True
+            _s["rem", "nto_pairs"] = True
+            _s["rem", "make_cube_files"] = "ntos"
+            _s["rem", "cubefile_state"] = i + 1
+            _s["plots", "comment"] = (
+                f"\n  {n_x} -10.0 10.0\n  {n_y} -10.0 10.0\n  {n_z} -10.0 10.0\n  {2*num_nto_pairs} 0 0 0\n  "
+                + " ".join(
+                    f"{n_alpha + i}"
+                    for i in range(-num_nto_pairs + 1, num_nto_pairs + 1)
+                )
+            )  # {n_alpha-2} {n_alpha-1} {n_alpha} {n_alpha+1} {n_alpha+2} {n_alpha+3}')
+            inp = inp + mtr.QChemInput(settings=_s)
+
+        with self.io() as io:
+            inp.write(io.inp)
+
+            self.engine.execute(self.io)
+
+            molecule.electronic_excitations = self.parse(io.out)
+
+        return molecule
+
 
 class QChemMinimizeKoopmanError(Task):
     def __init__(
@@ -936,103 +1042,95 @@ class QChemMinimizeKoopmanError(Task):
 
         with self.io() as io:
             if alpha is None:
-                [omega], J = mtr.MaxLIPOTR(_objective).run(
+                [omega, alpha], J = mtr.MaxLIPOTR(_objective).run(
                     x_min=[1e-3, 0], x_max=[1, 1 / epsilon], num_evals=num_evals
                 )
             else:
-                [omega, alpha], J = mtr.MaxLIPOTR(
+                [omega], J = mtr.MaxLIPOTR(
                     functools.partial(_objective, _alpha=alpha)
                 ).run(x_min=1e-3, x_max=1, num_evals=num_evals)
 
         return omega, alpha, J * mtr.eV
 
 
-class QChemMinimizeKoopmanErrorLPSCF(Task):
-    def __init__(
-        self,
-        engine: mtr.Engine,
-        io: mtr.IO,
-        handlers: Optional[Iterable[mtr.Handler]] = None,
-        name: Optional[str] = None,
-    ) -> None:
-        super().__init__(
-            (engine.num_threads or 1) * (engine.num_processors or 1),
-            handlers=handlers,
-            name=name,
-        )
-        self.engine = engine
-        self.io = io
+# class QChemMinimizeKoopmanErrorLPSCF(Task):
+#     def __init__(
+#         self,
+#         engine: mtr.Engine,
+#         io: mtr.IO,
+#         handlers: Optional[Iterable[mtr.Handler]] = None,
+#         name: Optional[str] = None,
+#     ) -> None:
+#         super().__init__(
+#             (engine.num_threads or 1) * (engine.num_processors or 1),
+#             handlers=handlers,
+#             name=name,
+#         )
+#         self.engine = engine
+#         self.io = io
 
-    def defaults(self, settings: mtr.Settings) -> mtr.Settings:
-        if ("rem", "basis") not in settings:
-            settings["rem", "basis"] = "3-21G"
-        if ("rem", "jobtype") not in settings:
-            settings["rem", "jobtype"] = "sp"
-        if ("rem", "exchange") not in settings:
-            settings["rem", "exchange"] = "gen"
-        if ("rem", "lrc_dft") not in settings:
-            settings["rem", "lrc_dft"] = True
-        if ("rem", "src_dft") not in settings:
-            settings["rem", "src_dft"] = 2
+#     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
+#         if ("rem", "basis") not in settings:
+#             settings["rem", "basis"] = "3-21G"
+#         if ("rem", "jobtype") not in settings:
+#             settings["rem", "jobtype"] = "sp"
+#         if ("rem", "exchange") not in settings:
+#             settings["rem", "exchange"] = "gen"
+#         if ("rem", "lrc_dft") not in settings:
+#             settings["rem", "lrc_dft"] = True
+#         if ("rem", "src_dft") not in settings:
+#             settings["rem", "src_dft"] = 2
 
-        return settings
+#         return settings
 
-    def run(
-        self,
-        *fragments: mtr.Structure,
-        active_fragment: int,
-        settings: Optional[mtr.Settings] = None,
-        epsilon: Optional[Union[int, float]] = 1.0,
-        alpha: Optional[float] = 0.2,
-        num_evals: Optional[int] = 5,
-    ) -> float:
-        beta = 1 / epsilon - alpha
+#     def run(
+#         self,
+#         *fragments: mtr.Structure,
+#         active_fragment: int,
+#         settings: Optional[mtr.Settings] = None,
+#         epsilon: Optional[Union[int, float]] = 1.0,
+#         alpha: Optional[float] = 0.2,
+#         num_evals: Optional[int] = 5,
+#     ) -> float:
+#         beta = 1 / epsilon - alpha
 
-        s = self.defaults(settings)
-        s["rem", "hf_sr"] = int(round(1000 * alpha))
-        s["rem", "hf_lr"] = int(round(1000 * (alpha + beta)))
-        s["xc_functional"] = (
-            ("X", "HF", alpha),
-            ("X", "wPBE", beta),
-            ("X", "PBE", 1 - alpha - beta),
-            ("C", "PBE", 1.0),
-        )
+#         s = self.defaults(settings)
+#         s["rem", "hf_sr"] = int(round(1000 * alpha))
+#         s["rem", "hf_lr"] = int(round(1000 * (alpha + beta)))
+#         s["xc_functional"] = (
+#             ("X", "HF", alpha),
+#             ("X", "wPBE", beta),
+#             ("X", "PBE", 1 - alpha - beta),
+#             ("C", "PBE", 1.0),
+#         )
 
-        with self.io() as io:
+#         with self.io() as io:
 
-            def f(omega):
-                omega = int(round(1000 * omega))
-                s["rem", "omega"] = s["rem", "omega2"] = omega
+#             def f(omega):
+#                 omega = int(round(1000 * omega))
+#                 s["rem", "omega"] = s["rem", "omega2"] = omega
 
-                wd = mtr.expand(f"{io.work_dir}/{omega}")
+#                 wd = mtr.expand(f"{io.work_dir}/{omega}")
 
-                gs_io = mtr.IO("gs.in", "gs.out", wd)
-                cation_io = mtr.IO("cation.in", "cation.out", wd)
-                anion_io = mtr.IO("anion.in", "anion.out", wd)
+#                 gs_io = mtr.IO("gs.in", "gs.out", wd)
+#                 cation_io = mtr.IO("cation.in", "cation.out", wd)
+#                 anion_io = mtr.IO("anion.in", "anion.out", wd)
 
-                ke = mtr.QChemKoopmanErrorLPSCF(self.engine, gs_io, cation_io, anion_io)
-                # FIXME: not sure the best way to handle num_consumers here...
-                return ke.run(
-                    *fragments,
-                    active_fragment=active_fragment,
-                    settings=settings,
-                    num_consumers=3,
-                )
+#                 ke = mtr.QChemKoopmanErrorLPSCF(self.engine, gs_io, cation_io, anion_io)
+#                 # FIXME: not sure the best way to handle num_consumers here...
+#                 return ke.run(
+#                     *fragments,
+#                     active_fragment=active_fragment,
+#                     settings=settings,
+#                     num_consumers=3,
+#                 )
 
-            return mtr.MaxLIPOTR(f).run(x_min=1e-3, x_max=1, num_evals=num_evals)
+#             return mtr.MaxLIPOTR(f).run(x_min=1e-3, x_max=1, num_evals=num_evals)
 
 
 class QChemOptimize(QChemBaseTask):
     def parse(self, output: str) -> Any:
-        try:
-            parsed = cclib.io.ccread(output)
-            coords = parsed.atomcoords[-1, :, :] * mtr.angstrom
-            zs = parsed.atomnos
-        except AttributeError:
-            return None
-
-        atoms = (mtr.Atom(element=Z, position=p) for Z, p in zip(zs, coords))
-        return mtr.Structure(*atoms)
+        return QChemOutput(output).structure
 
     def defaults(self, settings: mtr.Settings) -> mtr.Settings:
         if ("rem", "exchange") not in settings and ("rem", "method",) not in settings:
@@ -1051,6 +1149,24 @@ class QChemOptimize(QChemBaseTask):
         return molecule
 
 
+# def f(molecule, alpha, beta, omega):
+#     n_alpha = round((sum(molecule.atomic_numbers) + molecule.charge)/2)
+#     td_settings=mtr.Settings(rem=dict(basis='cc-pVTZ',exchange='gen',cis_n_roots=20,hf_sr=int(round(1000*alpha)),hf_lr=int(round(1000*(alpha+beta))),omega=int(round(1000*omega)),omega2=int(round(1000*omega)),lrc_dft=True,src_dft=2,rpa=True,cis_singlets=True,cis_triplets=False),xc_functional=(('X','HF',alpha),('X','wPBE',beta),('X','PBE',1 - alpha - beta),('C','PBE',1.0)))
+
+#     inps = [copy.deepcopy(td_settings) for _ in range(1,td_settings['rem','cis_n_roots']+1)]
+#     for i,inp in enumerate(inps):
+#         inp['rem','scf_guess'] = 'read'
+#         inp['rem','skip_cis_rpa'] = True
+#         inp['rem','nto_pairs'] = True
+#         inp['rem','make_cube_files'] = 'ntos'
+#         inp['rem','cubefile_state'] = i+1
+#         inp['plots','comment'] = f'\n  200 -10.0 10.0\n  200 -10.0 10.0\n  200 -10.0 10.0\n  6 0 0 0\n  {n_alpha-2} {n_alpha-1} {n_alpha} {n_alpha+1} {n_alpha+2} {n_alpha+3}'
+#     return '\n@@@\n\n'.join(str(mtr.QChemInput(settings=inp)) for inp in inps)
+
+# def repl(m):
+#     return '$molecule\n  read\n$end\n'
+
+
 class QChemPolarizability(QChemBaseTask):
     def parse(self, output: str) -> Any:
         try:
@@ -1285,7 +1401,7 @@ class QChemSinglePointFrontier(QChemBaseTask):
 
 #     def run(
 #         self,
-#         structure: Union[mtr.QChemStructure, mtr.QChemFragments, mtr.Structure],
+#         structure: Union[mtr.QChemStructure, mtr. , mtr.Structure],
 #         settings: Optional[mtr.Settings] = None,
 #     ) -> None:
 #         s = mtr.Settings() if settings is None else copy.deepcopy(settings)
